@@ -2,7 +2,7 @@
    miscutil.cpp : This file is part of pstoedit
    misc utility functions
 
-   Copyright (C) 1998 - 2018  Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1998 - 2023  Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,11 @@
 
 */
 #include "miscutil.h"
+#include <memory>
+
+#ifdef OS_WIN32_WCE
+#include "WinCEAdapter.h"
+#endif
 
 #include I_stdio
 
@@ -33,15 +38,23 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-
 #ifdef __hpux
 // HP-UX does not define getcwd in unistd.h
 extern "C"  char *getcwd(char *, size_t);
 #endif
-
+#elif defined(OS_WIN32_WCE)
+#include <windows.h>
+#elif defined(_WIN32)
+#include <windows.h>
+#include <direct.h>
+#elif defined (__OS2__)
+#define INCL_DOS
+#define INCL_WINWINDOWMGR
+#define INCL_WINSHELLDATA		/* Window Shell functions       */
+#define INCL_WINERRORS
+#include <os2.h>
 #else
-// not Unix like
-
+// not Unix / winwdows like
 #if HAVE_DIRENT_H
 #include <dirent.h>
 #elif HAVE_SYS_NDIR_H
@@ -50,11 +63,7 @@ extern "C"  char *getcwd(char *, size_t);
 #include <sys/dir.h>
 #elif HAVE_NDIR_H
 #include <ndir.h>
-#else
-// last chance
-#include <direct.h>
 #endif
-
 #endif
 
 #ifdef HAVE_MKSTEMP
@@ -63,21 +72,8 @@ extern "C"  char *getcwd(char *, size_t);
 #endif
 
 #include I_stdlib
-
 #include I_string_h
 
-
-#if defined(_WIN32)
-#include <windows.h>
-#elif defined (__OS2__)
-#define INCL_DOS
-#define INCL_WINWINDOWMGR
-#define INCL_WINSHELLDATA		/* Window Shell functions       */
-#define INCL_WINERRORS
-#include <os2.h>
-#endif
-
-
 #if defined(unix) || defined(__unix__) || defined(_unix) || defined(__unix) || defined (NetBSD) 
 //take this out (we may have backslashes w/ EMX(OS/2)):  || defined(__EMX__)
 void convertBackSlashes(char *) { }
@@ -87,7 +83,7 @@ void convertBackSlashes(char *) { }
 void convertBackSlashes(char *fileName)
 {
  	char *c ;
-	while ((c = strchr(fileName, '\\')) != NIL)
+	while ((c = strchr(fileName, '\\')) != nullptr)
 		*c = '/';
 }
 #endif
@@ -168,7 +164,7 @@ char *tempnam(const char *, const char *pfx)
  static const char * testvar(const char * var) {
 	const char * res = getenv(var);
 	if (res && dirAccessible(res)) return res;
-	else return 0;
+	else return nullptr;
  }
 
 #endif
@@ -176,12 +172,12 @@ char *tempnam(const char *, const char *pfx)
 RSString full_qualified_tempnam(const char *pref)
 {
 #ifdef HAVE_MKSTEMP
-	const char *path = 0;
-	char * filename = 0;
+	const char *path = nullptr;
+	char * filename = nullptr;
 	const char XXXXXX[] = "XXXXXX" ; // needed for mkstemp template
-	(void) ((path = testvar("TEMP"))   == 0L && 
-		(path = testvar("TMP"))    == 0L &&
-		(path = testvar("TMPDIR")) == 0L &&
+	(void) ((path = testvar("TEMP"))   == nullptr && 
+		(path = testvar("TMP"))    == nullptr &&
+		(path = testvar("TMPDIR")) == nullptr &&
 		(path = dirAccessible("/tmp" )  ? "/tmp" : "." )  // last resort current dir
 		);
 	const unsigned int needed = strlen(path) + 1 + strlen(pref) + 1 + strlen(XXXXXX) + 2;
@@ -204,12 +200,7 @@ RSString full_qualified_tempnam(const char *pref)
 	}
 
 #else 
-#if defined (__BCPLUSPLUS__) || defined (__TCPLUSPLUS__)
-/* borland has a prototype that expects a char * as second arg */
-	char *filename = TEMPNAM(0, (char *) pref);
-#else
-	char *filename = TEMPNAM(0, pref);
-#endif
+	char *filename = TEMPNAM(nullptr, pref);
 	// W95: Fkt. tempnam() erzeugt Filename+Pfad
 	// W3.1: es wird nur der Name zurueckgegeben
 #endif
@@ -221,27 +212,25 @@ RSString full_qualified_tempnam(const char *pref)
 	return result;
 #else
 	convertBackSlashes(filename);
-	if ((strchr(filename, '\\') == 0) && (strchr(filename, '/') == 0)) {	// keine Pfadangaben..
+	RSString result("");
+	if ((strchr(filename, '\\') == nullptr) && (strchr(filename, '/') == nullptr)) {	// keine Pfadangaben..
 		char cwd[400];
-		(void) GETCWD(cwd, 400);
-		RSString result = cwd;
-		result += "/";
-		result += filename;
-		free(filename);
-		return result;
-	} else {
-		const RSString result(filename);
-		free(filename);
-		return result;
-	}
+		if (GETCWD(cwd, 400)) {
+		    result += cwd;
+			result += "/";
+		}
+	} 
+	result += filename;
+	free(filename);
+	return result;
 #endif
 }
 
-unsigned short hextoint(const char hexchar) 
+unsigned short hextoint(const char hexchar)
 {
 	char h = hexchar;
 	if (h >= 'a' && h <='f') h += 'A' - 'a'; // normalize lowercase to uppercase
-	unsigned short r = ( h <= '9' ) ? (h - '0') : (h + 10 - 'A' ) ; //lint !e732
+	const unsigned short r = ( h <= '9' ) ? (h - '0') : (h + 10 - 'A' ) ; //lint !e732
 	return r;
 }
 #if defined(_WIN32)
@@ -251,12 +240,18 @@ RSString tryregistry(HKEY hKey, LPCSTR subkeyn, LPCSTR key)
 {
 	HKEY subkey;
 	static RSString emptyString("");
+
 	const long ret = RegOpenKeyEx(hKey,	// HKEY_LOCAL_MACHINE, //HKEY hKey,
+#ifdef OS_WIN32_WCE
+								  LPSTRtoLPWSTR(subkeyn).c_str(),	// LPCSTR lpSubKey,
+#else
 								  subkeyn,	// LPCSTR lpSubKey,
+#endif
 								  0L,	// DWORD ulOptions,
 								  KEY_READ,	// REGSAM samDesired,
 								  &subkey	//PHKEY phkResult
-		);
+								  );
+
 	if (ret != ERROR_SUCCESS) {
 		if (regdebug) cerr << "RegOpenKeyEx :" <<subkeyn << ":" << key << ": failed with error code " << ret << endl;
 		return emptyString;
@@ -266,13 +261,19 @@ RSString tryregistry(HKEY hKey, LPCSTR subkeyn, LPCSTR key)
 		BYTE value[maxvaluelength];
 		DWORD bufsize = maxvaluelength;
 		DWORD valuetype;
+
 		const long retv = RegQueryValueEx(subkey,	// HKEY_LOCAL_MACHINE, //HKEY hKey,
+#ifdef OS_WIN32_WCE
+										  LPSTRtoLPWSTR(key).c_str(),	// "SOFTWARE\\wglunz\\pstoedit\\plugindir", //LPCSTR lpValueName,
+#else
 										  key,	// "SOFTWARE\\wglunz\\pstoedit\\plugindir", //LPCSTR lpValueName,
-										  NIL,	// LPDWORD lpReserved,
+#endif
+										  nullptr,	// LPDWORD lpReserved,
 										  &valuetype,	// LPDWORD lpType,
 										  value,	// LPBYTE lpData,
 										  &bufsize	// LPDWORD lpcbData
 			);
+
 		(void) RegCloseKey(subkey);
 		if (retv != ERROR_SUCCESS) {
 			if (regdebug)  cerr << "RegQueryValueEx :" <<subkeyn << ":" << key << ": failed with error code " << retv << endl;
@@ -322,7 +323,7 @@ RSString getRegistryValue(ostream & errstream, const char *typekey, const char *
 
 	getini(0, errstream, pszFileName, inifilename, (int) sizeof(pszFileName));
 	hini = PrfOpenProfile(hab, pszFileName);
-	rc = PrfQueryProfileString(hini, typekey, key, NIL, (PVOID) buffer, (LONG) sizeof(buffer));
+	rc = PrfQueryProfileString(hini, typekey, key, nullptr, (PVOID) buffer, (LONG) sizeof(buffer));
 	PrfCloseProfile(hini);
 	WinTerminate(hab);
 
@@ -559,7 +560,13 @@ size_t searchinpath(const char *EnvPath, const char *name,
 
 unsigned long P_GetPathToMyself(const char *name, char *returnbuffer, unsigned long buflen)
 {
-#if defined(_WIN32)
+#if defined (OS_WIN32_WCE)
+	wchar_t wszReturnBuffer[MAX_PATH] = L"";
+	unsigned long ulReturn =  GetModuleFileName(GetModuleHandle(LPSTRtoLPWSTR(name).c_str()), wszReturnBuffer, MAX_PATH);
+	wszReturnBuffer[MAX_PATH-1] = L'\0';
+	BSS_UTI_WCharToAscii(wszReturnBuffer, returnbuffer, buflen);
+	return ulReturn;
+#elif defined(_WIN32)
 	return GetModuleFileName(GetModuleHandle(name), returnbuffer, buflen);
 #elif defined (__OS2__)
 	PTIB pptib;
@@ -587,7 +594,7 @@ unsigned long P_GetPathToMyself(const char *name, char *returnbuffer, unsigned l
 void errorMessage(const char *errortext)
 {
 #if 0 // defined(_WIN32)
-	MessageBox(NIL, errortext, "pstoedit", MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
+	MessageBox(nullptr, errortext, "pstoedit", MB_OK | MB_ICONEXCLAMATION | MB_TASKMODAL);
 #else
 	cerr << errortext << endl;
 #endif
@@ -599,11 +606,11 @@ DLLEXPORT RSString getOutputFileNameFromPageNumber(const char * const outputFile
 	const char PAGENUMBER_String[] = "%PAGENUMBER%";
 	const char * pagestringptr_1 = strstr(outputFileTemplate,PAGENUMBER_String);
 	const char * pagestringptr_2 = strstr(outputFileTemplate,"%d");
-	if ((pagestringptr_1 == NIL) && (pagestringptr_2 == NIL)) {
+	if ((pagestringptr_1 == nullptr) && (pagestringptr_2 == nullptr)) {
 		return RSString(outputFileTemplate);
 	} else  {
 		const size_t size = strlen(outputFileTemplate) + 30;
-		char * newname = new char[ size ];
+		std::unique_ptr<char[]> newname ( new char[ size ]);
 
 		RSString formatting("%");
 		formatting += pagenumberformatOption;
@@ -620,22 +627,21 @@ DLLEXPORT RSString getOutputFileNameFromPageNumber(const char * const outputFile
 
 		if (pagestringptr_1) // preference to %PAGENUMBER%
 		{
-			strncpy_s(newname,size,outputFileTemplate,pagestringptr_1-outputFileTemplate); // copy up to %PAGENUMBER%
+			strncpy_s(newname.get(),size,outputFileTemplate,pagestringptr_1-outputFileTemplate); // copy up to %PAGENUMBER%
 			// now copy page number
-			strcat_s(newname,size,pagenumberstring);
+			strcat_s(newname.get(),size,pagenumberstring);
 			// now copy trailer
-			strcat_s(newname,size,pagestringptr_1+strlen(PAGENUMBER_String));
+			strcat_s(newname.get(),size,pagestringptr_1+strlen(PAGENUMBER_String));
 
 		} else {
-			strncpy_s(newname,size,outputFileTemplate,pagestringptr_2-outputFileTemplate); // copy up to %PAGENUMBER%
+			strncpy_s(newname.get(),size,outputFileTemplate,pagestringptr_2-outputFileTemplate); // copy up to %PAGENUMBER%
 			// now copy page number
-			strcat_s(newname,size,pagenumberstring);
+			strcat_s(newname.get(),size,pagenumberstring);
 			// now copy trailer
-			strcat_s(newname,size,pagestringptr_2+strlen("%d"));		
+			strcat_s(newname.get(),size,pagestringptr_2+strlen("%d"));
 		}
 		//cout << "newname " << newname << endl;
-		const RSString result (newname);
-		delete [] newname;
+		const RSString result (newname.get());
 		return result;
 	}
 }
@@ -767,19 +773,8 @@ void RSString::assign(const char *src, const size_t len )
 
 bool fileExists(const char *filename)
 {
-#ifdef HAVE_STL
 	std::ifstream test(filename);
 	return test.is_open();
-#else
-#if defined (__GNUG__)  && (__GNUC__>=3)
-	ifstream test(filename, ios::in );	//lint !e655
-#else
-	ifstream test(filename, ios::in | ios::nocreate);	//lint !e655
-	// MSVC needs ios::nocreate if used for testing whether file exists
-	// also others except the g++ > 3.0 accept this
-#endif
-	return (test ? true : false);
-#endif
 }
 
 
@@ -797,7 +792,7 @@ static void skipws(char *&lineptr)
 
 static char *readword(char *&lineptr)
 {
-	char *result = 0;
+	char *result = nullptr;
 	if (*lineptr == '"') {
 		result = strtok(lineptr, "\"");
 	} else {
@@ -823,7 +818,7 @@ void FontMapper::readMappingTable(ostream & errstream, const char *filename)
 	while (!inFile.getline(line, linesize).eof()) {
 		linenr++;
 		strcpy_s(save,linesize, line);
-#ifdef HAVE_STL
+ 
 		// Notes regarding ANSI C++ version (from KB)
 		// istream::get( char* pch, int nCount, char delim ) is different in three ways: 
 		// When nothing is read, failbit is set.
@@ -836,7 +831,7 @@ void FontMapper::readMappingTable(ostream & errstream, const char *filename)
 			inFile.clear();
 			continue;
 		}
-#endif
+
 		if (line[0] == '%')
 			continue;
 		char *lineptr = line;
@@ -848,15 +843,15 @@ void FontMapper::readMappingTable(ostream & errstream, const char *filename)
 		//	errstream << "empty line in fontmap - ignored " << endl;
 			continue;
 		}
-		char *original = readword(lineptr);
+		const char * const original = readword(lineptr);
 		skipws(lineptr);
-		char *replacement = readword(lineptr);
+		const char * const replacement = readword(lineptr);
 		if (original && replacement) {
 			// errstream << "\"" << original << "\" \"" << replacement <<"\""<< endl;
 		        if (replacement[0] == '/') {
 				// Map to an existing entry.
 			  	const RSString * prevEntry = getValue(replacement+1);
-				if (prevEntry == NULL)
+				if (prevEntry == nullptr)
 					errstream << "undefined font " <<
 					  replacement+1 <<
 					  " found in line (" << linenr <<
@@ -911,10 +906,10 @@ const char *FontMapper::mapFont(const RSString & fontname) const
 				if (r)
 			        return r->c_str();
 				else 
-					return 0;
+					return nullptr;
 			}
 		}
-		return 0;
+		return nullptr;
     }
 }
 
@@ -923,19 +918,21 @@ unsigned int Argv::parseFromString(const char * const argstring) {
 	const char * cp = argstring;
 	while (cp && *cp) { // for all args
 		while (*cp == ' ') cp++; // skip leading space
-		RSString result("");
+		if (!*cp) break;
+		RSString result("");		
 		if (*cp == '"')	{ // handle string arg - read everything until closing "
-				cp++; // skip leading "
-				while (*cp && (*cp != '"')) {
-					result += *cp; 
-					cp++;
-				}
-				if (*cp) cp++; // skip trailing "
+			cp++; // skip leading "
+			while (*cp && (*cp != '"')) {
+				result += *cp; 
+				cp++;
+			}
+			if (*cp) cp++; // skip trailing "
 		} else {
-				while (*cp && (*cp != ' ')) {
-					result += *cp; 
-					cp++;
-				}
+			while (*cp && (*cp != ' ')) {
+				result += *cp; 
+				cp++;
+			}
+			assert(result.length() > 0);
 		}
 		addarg(result);
 		nrOfNewArgs++;
